.k-rad-contentdiv {
    width: 100%;
    height: 100%;
}

#rad-cps {
    height: 100% !important;
}

.k-rad-spectrum-canvas {
    width: 100%;
    min-height: 200px;
    height: 200px;
}

.k-rad-cps-canvas {
    width: 100%;
    height: 250px;
    max-height: 250px;
}

.k-rad-usv-canvas {
    width: 100%;
    height: 250px;
    max-height: 250px;
}

.k-rad-graph-title {
    font-size: large;
    display: flex;
    align-items: center;
    justify-content: center;
}

.k-rad-contentdiv {
    width: 100%;
    height: 100%;
}

.radiation-toolbar-cps {
    margin-left: 2px;
    margin-right: 2px;
}

.radiation-toolbar-usv {
    margin-left: 2px;
    margin-right: 2px;
}

.rad-side {
    display: flex !important;
    flex-direction: row !important;
}

.rad-dual-stack {
    display: flex;
    flex-direction: column;

    margin-left: 2px;
    margin-right: 2px;

    align-items: center;
    justify-content: center;
}

.rad-flex-box {
    margin: 10px;
}

.rad-flex-stack {
    display: flex;
    flex-direction: column;
    gap: 10px;
}

.rad-flex-row {
    display: flex;
    flex-direction: row;
    gap: 10px;
}

.rad-flex-header {
    flex: 0.25;
    font-weight: bold;
}