div.k-s-container {
    height: 100%;
    width: 100%;

    display: flex;
    flex-direction: row;
}

div.k-s-list {
    height: 100%;
    overflow-x: hidden;
    overflow-y: scroll;
    padding-top: 10px;
    padding-bottom: 10px;

    display: flex;
    flex-direction: column;

    flex: "25%";
}

div.k-s-pane-holder {
    display: flex;
    flex-direction: column;
    height: 100%;
    width: 75%;
    flex: 1;
}

div.k-s-pane-content {
    height: 100%;
    overflow-x: hidden;
    overflow-y: auto;
    flex: 1;
    padding: 10px;
}

div.k-s-pane-buttons {
    height: 35px;
    text-align: right;
    padding-right: 20px;
}

div.k-s-list-item {
    padding-top: 2px;
    padding-bottom: 2px;
    padding-left: 10px;
    padding-right: 10px;
}

div.k-s-list-item:hover {
    background-color: #C0C0C0;
    cursor: pointer;
}

div.k-s-list-item-active {
    background-color: #D0D0D0;
}

div.k-s-alert {
    padding-left: 10px;
    padding-top: 5px;
    display: flex;
    flex-direction: column;
    height: 100%;
}

div.k-s-alert-content {
    flex: 1;
    height: 100%;
}

div.k-s-alert-header {
    font-size: 18px;
    font-weight: bold;
}

div.k-s-list-div {
    margin-top: 5px;
    margin-bottom: 5px;
}

