#channels {
    overflow: hidden !important;
    position: relative;
    width: 100%;
    height: 100%;
}

.k_cd_container {
    position: absolute;

    top: 24px;
    left: 0px; 
    right: 0px;
    bottom: 0px;
}

.k_cd_banner {
    position: absolute;

    left: 0px;
    top: 0px;
    right: 0px;

    height: 22px;
    max-height: 22px;

    margin-top: 2px;
    margin-bottom: 2px;

    width: 100%;

    overflow: hidden !important;

    background-color: #d0d0d0;

    font-size: 10px;
}

.k_cd_banner select {
    margin-left: 5px;
    margin-right: 5px;
}

.k_cd_picker {
    position: absolute;
    right: 10px;
    top: 5px;
    z-index: 100;

    display: flex;
    justify-content: center;
    align-items:center;

}

.k_cd_type {
    position: absolute;
    left: 10px;
    top: 5px;
    z-index: 100;

    display: flex;
    justify-content: center;
    align-items:center;


    padding-left: 5px;
    padding-right: 5px;
}

.k_cd_type select {

}

